@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software
-  Foundation, Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free
+  Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -88,6 +88,7 @@ void intr (int sig);
 void lcd (int, char **);
 int login (char *);
 void lostpeer (int sig);
+void lpwd (int, char **);
 void ls (int, char **);
 void mabort (int);
 void macdef (int, char **);
@@ -104,7 +105,7 @@ void proxabort (int sig);
 void proxtrans (char *, char *, char *);
 void psabort (int sig);
 void pswitch (int);
-void ptransfer (char *, long, struct timeval *, struct timeval *);
+void ptransfer (char *, long long, struct timeval *, struct timeval *);
 void put (int, char **);
 void pwd (int, char **);
 void quit (int, char **);
@@ -151,7 +152,6 @@ void setverbose (int, char **);
 void shell (int, char **);
 void site (int, char **);
 void sizecmd (int, char **);
-char *slurpstring (void);
 void status (int, char **);
 void syst (int, char **);
 void tvsub (struct timeval *, struct timeval *, struct timeval *);
