@@ -1,7 +1,7 @@
 /* Header for poll(2) emulation
    Contributed by Paolo Bonzini.
 
-   Copyright 2001-2003, 2007, 2009-2012 Free Software Foundation, Inc.
+   Copyright 2001-2003, 2007, 2009-2013 Free Software Foundation, Inc.
 
    This file is part of gnulib.
 
@@ -16,8 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License along
-   with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 #ifndef _@GUARD_PREFIX@_POLL_H
 
@@ -45,7 +44,7 @@
 /* fake a poll(2) environment */
 # define POLLIN      0x0001      /* any readable data available   */
 # define POLLPRI     0x0002      /* OOB/Urgent readable data      */
-# define POLLOUT     0x0004      /* file descriptor is writeable  */
+# define POLLOUT     0x0004      /* file descriptor is writable   */
 # define POLLERR     0x0008      /* some poll error occurred      */
 # define POLLHUP     0x0010      /* file descriptor was "hung up" */
 # define POLLNVAL    0x0020      /* requested events "invalid"    */
