@@ -1,6 +1,6 @@
 /*
   Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-  2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software
+  2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free Software
   Foundation, Inc.
 
   This file is part of GNU Inetutils.
@@ -70,7 +70,8 @@
 #ifndef __MISC_PROTO__
 # define __MISC_PROTO__
 
-void auth_encrypt_init (char *, char *, char *, int);
+void auth_encrypt_init (char *, char *, char *, char *, int);
+void auth_encrypt_user (char *);
 void auth_encrypt_connect (int);
 void printd (unsigned char *, int);
 
