@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012 Free Software
+  Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free Software
   Foundation, Inc.
 
   This file is part of GNU Inetutils.
@@ -30,7 +30,10 @@
 #include <netinet/in.h>
 #include <netinet/ip.h>
 /* #include <netinet/ip_icmp.h> -- Deliberately not including this
-   since the definitions are are using are pulled in by libicmp. */
+   since the definitions in use are being pulled in by libicmp. */
+#ifdef HAVE_NETINET_IP_VAR_H
+# include <netinet/ip_var.h>
+#endif
 
 #include <arpa/inet.h>
 #include <netdb.h>
@@ -46,7 +49,14 @@
 #include <limits.h>
 #include <assert.h>
 #include <argp.h>
+#include <unused-parameter.h>
 #include <icmp.h>
+#ifdef HAVE_LOCALE_H
+# include <locale.h>
+#endif
+#ifdef HAVE_IDNA_H
+# include <idna.h>
+#endif
 
 #include "xalloc.h"
 #include "libinetutils.h"
@@ -72,10 +82,11 @@ typedef struct trace
 
 void trace_init (trace_t * t, const struct sockaddr_in to,
 		 const enum trace_type type);
+void trace_ip_opts (struct sockaddr_in *to);
 void trace_inc_ttl (trace_t * t);
 void trace_inc_port (trace_t * t);
 void trace_port (trace_t * t, const unsigned short port);
-int trace_read (trace_t * t);
+int trace_read (trace_t * t, int * type, int * code);
 int trace_write (trace_t * t);
 int trace_udp_sock (trace_t * t);
 int trace_icmp_sock (trace_t * t);
@@ -88,16 +99,33 @@ void do_try (trace_t * trace, const int hop,
 char *get_hostname (struct in_addr *addr);
 
 int stop = 0;
-int pid = 0;
+int pid;
+int seqno;	/* Most recent sequence number.  */
 static char *hostname = NULL;
 char addrstr[INET6_ADDRSTRLEN];
 struct sockaddr_in dest;
 
-static enum trace_type opt_type = TRACE_ICMP;
-int opt_port = 33434;
+#ifdef IP_OPTIONS
+size_t len_ip_opts = 0;
+char ip_opts[MAX_IPOPTLEN];
+#endif /* IP_OPTIONS */
+
+/* Cause for destination unreachable reply,
+ * encoded as a single character.
+ */
+const char unreach_sign[NR_ICMP_UNREACH + 2] = "NHPPFS**U**TTXXX";
+
+static enum trace_type opt_type = TRACE_UDP;
+int opt_port = TRACE_UDP_PORT;
 int opt_max_hops = 64;
 static int opt_max_tries = 3;
 int opt_resolve_hostnames = 0;
+int opt_tos = -1;	/* Triggers with non-negative values.  */
+int opt_ttl = TRACE_TTL;
+int opt_wait = TIME_INTERVAL;
+#ifdef IP_OPTIONS
+char *opt_gateways = NULL;
+#endif
 
 const char args_doc[] = "HOST";
 const char doc[] = "Print the route packets trace to network host.";
@@ -113,15 +141,26 @@ enum {
 
 static struct argp_option argp_options[] = {
 #define GRP 0
+  {"first-hop", 'f', "NUM", 0, "set initial hop distance, i.e., time-to-live",
+   GRP+1},
+#ifdef IP_OPTIONS
+  {"gateways", 'g', "GATES", 0, "list of gateways for loose source routing",
+   GRP+1},
+#endif
+  {"icmp", 'I', NULL, 0, "use ICMP ECHO as probe", GRP+1},
+  {"max-hop", 'm', "NUM", 0, "set maximal hop count (default: 64)", GRP+1},
   {"port", 'p', "PORT", 0, "use destination PORT port (default: 33434)",
    GRP+1},
   {"resolve-hostnames", OPT_RESOLVE, NULL, 0, "resolve hostnames", GRP+1},
+  {"tos", 't', "NUM", 0, "set type of service (TOS) to NUM", GRP+1},
   {"tries", 'q', "NUM", 0, "send NUM probe packets per hop (default: 3)",
    GRP+1},
   {"type", 'M', "METHOD", 0, "use METHOD (`icmp' or `udp') for traceroute "
-   "operations", GRP+1},
+   "operations, defaulting to `udp'", GRP+1},
+  {"wait", 'w', "NUM", 0, "wait NUM seconds for response (default: 3)",
+   GRP+1},
 #undef GRP
-  {NULL}
+  {NULL, 0, NULL, 0, NULL, 0}
 };
 
 static error_t
@@ -132,14 +171,47 @@ parse_opt (int key, char *arg, struct argp_state *state)
 
   switch (key)
     {
-    case 'p':
-      opt_port = strtoul (arg, &p, 0);
-      if (*p || opt_port == 0 || opt_port > 65536)
-        error (EXIT_FAILURE, 0, "invalid port number `%s'", arg);
+    case 'f':
+      opt_ttl = strtol (arg, &p, 0);
+      if (*p || opt_ttl <= 0 || opt_ttl > 255)
+        error (EXIT_FAILURE, 0, "impossible distance `%s'", arg);
       break;
 
-    case OPT_RESOLVE:
-      opt_resolve_hostnames = 1;
+#ifdef IP_OPTIONS
+    case 'g':
+      if (opt_gateways)
+	{
+	  size_t len = 0;
+
+	  len = strlen (opt_gateways);
+	  opt_gateways = xrealloc (opt_gateways, len + strlen (arg) + 3);
+
+	  /* Append new gateways to old list,
+	   * separating the two parts by a comma.
+	   */
+	  opt_gateways[len] = ',';
+	  opt_gateways[len + 1] = '\0';
+	  strcat (opt_gateways, arg);
+	}
+      else
+	opt_gateways = xstrdup (arg);
+      break;
+#endif /* IP_OPTIONS */
+
+    case 'I':
+      opt_type = TRACE_ICMP;
+      break;
+
+    case 'm':
+      opt_max_hops = strtol (arg, &p, 0);
+      if (*p || opt_max_hops <= 0 || opt_max_hops > 255)
+	error (EXIT_FAILURE, 0, "invalid hops value `%s'", arg);
+      break;
+
+    case 'p':
+      opt_port = strtol (arg, &p, 0);
+      if (*p || opt_port <= 0 || opt_port > 65536)
+        error (EXIT_FAILURE, 0, "invalid port number `%s'", arg);
       break;
 
     case 'q':
@@ -150,6 +222,12 @@ parse_opt (int key, char *arg, struct argp_state *state)
         error (EXIT_FAILURE, 0, "number of tries should be between 1 and 10");
       break;
 
+    case 't':
+      opt_tos = strtol (arg, &p, 0);
+      if (*p || opt_tos < 0 || opt_tos > 255)
+	error (EXIT_FAILURE, 0, "invalid TOS value `%s'", arg);
+      break;
+
     case 'M':
       if (strcmp (arg, "icmp") == 0)
         opt_type = TRACE_ICMP;
@@ -159,6 +237,16 @@ parse_opt (int key, char *arg, struct argp_state *state)
         argp_error (state, "invalid method");
       break;
 
+    case 'w':
+      opt_wait = strtol (arg, &p, 0);
+      if (*p || opt_wait < 0 || opt_wait > 60)
+	error (EXIT_FAILURE, 0, "ridiculous waiting time `%s'", arg);
+      break;
+
+    case OPT_RESOLVE:
+      opt_resolve_hostnames = 1;
+      break;
+
     case ARGP_KEY_ARG:
       host_is_given = true;
       hostname = xstrdup(arg);
@@ -176,27 +264,57 @@ parse_opt (int key, char *arg, struct argp_state *state)
   return 0;
 }
 
-static struct argp argp = {argp_options, parse_opt, args_doc, doc};
+static struct argp argp =
+  {argp_options, parse_opt, args_doc, doc, NULL, NULL, NULL};
 
 int
 main (int argc, char **argv)
 {
+  int hop, rc;
+  char *rhost;
   struct addrinfo hints, *res;
   trace_t trace;
 
   set_program_name (argv[0]);
 
+#ifdef HAVE_SETLOCALE
+  setlocale (LC_ALL, "");
+#endif
+
+  pid = getpid() & 0xffff;
+
   /* Parse command line */
   iu_argp_init ("traceroute", program_authors);
   argp_parse (&argp, argc, argv, 0, NULL, NULL);
 
+  if ((hostname == NULL) || (*hostname == '\0'))
+    error (EXIT_FAILURE, 0, "unknown host");
+
   /* Hostname lookup first for better information */
   memset (&hints, 0, sizeof (hints));
   hints.ai_family = AF_INET;
   hints.ai_flags = AI_CANONNAME;
+#ifdef AI_IDN
+  hints.ai_flags |= AI_IDN;
+# ifdef AI_CANONIDN
+  hints.ai_flags |= AI_CANONIDN;
+# endif
+#endif
 
-  if ((hostname == NULL) || (*hostname == '\0')
-      || getaddrinfo (hostname, NULL, &hints, &res))
+#ifdef HAVE_IDN
+  rc = idna_to_ascii_lz (hostname, &rhost, 0);
+  free (hostname);
+
+  if (rc)
+    error (EXIT_FAILURE, 0, "unknown host");
+#else /* !HAVE_IDN */
+  rhost = hostname;
+#endif
+
+  rc = getaddrinfo (rhost, NULL, &hints, &res);
+  free (rhost);
+
+  if (rc)
     error (EXIT_FAILURE, 0, "unknown host");
 
   memcpy (&dest, res->ai_addr, res->ai_addrlen);
@@ -210,13 +328,17 @@ main (int argc, char **argv)
 
   freeaddrinfo (res);
 
+  trace_ip_opts (&dest);
+
   trace_init (&trace, dest, opt_type);
 
-  int hop = 1;
+  hop = 1;
+  seqno = -1;	/* One less than first usable packet number 0.  */
+
   while (!stop)
     {
       if (hop > opt_max_hops)
-	exit (EXIT_SUCCESS);
+	exit (EXIT_FAILURE);
       do_try (&trace, hop, opt_max_hops, opt_max_tries);
       trace_inc_ttl (&trace);
       trace_inc_port (&trace);
@@ -228,7 +350,8 @@ main (int argc, char **argv)
 
 void
 do_try (trace_t * trace, const int hop,
-	const int max_hops, const int max_tries)
+	const int max_hops _GL_UNUSED_PARAMETER,
+	const int max_tries)
 {
   fd_set readset;
   int ret, tries, readonly = 0;
@@ -236,35 +359,46 @@ do_try (trace_t * trace, const int hop,
   double triptime = 0.0;
   uint32_t prev_addr = 0;
 
-  printf (" %d  ", hop);
+  printf (" %2d  ", hop);
 
   for (tries = 0; tries < max_tries; tries++)
     {
-      FD_ZERO (&readset);
-      FD_SET (trace_icmp_sock (trace), &readset);
+      int save_errno;
+      int fd = trace_icmp_sock (trace);
 
-      time.tv_sec = TIME_INTERVAL;
+      FD_ZERO (&readset);
+      FD_SET (fd, &readset);
+
+      memset (&time, 0, sizeof (time));		/* 64-bit issue.  */
+      time.tv_sec = opt_wait;
       time.tv_usec = 0;
 
       if (!readonly)
 	trace_write (trace);
 
-      ret = select (FD_SETSIZE, &readset, NULL, NULL, &time);
+      errno = 0;
+      ret = select (fd + 1, &readset, NULL, NULL, &time);
+      save_errno = errno;
 
       gettimeofday (&now, NULL);
 
       now.tv_usec -= trace->tsent.tv_usec;
       now.tv_sec -= trace->tsent.tv_sec;
+      if (now.tv_usec < 0)
+	{
+	  --now.tv_sec;
+	  now.tv_usec += 1000000;
+	}
 
       if (ret < 0)
 	{
-	  switch (errno)
+	  switch (save_errno)
 	    {
 	    case EINTR:
 	      /* was interrupted */
 	      break;
 	    default:
-              error (EXIT_FAILURE, EPERM, "select failed");
+              error (EXIT_FAILURE, errno, "select failed");
 	      break;
 	    }
 	}
@@ -276,12 +410,16 @@ do_try (trace_t * trace, const int hop,
 	}
       else
 	{
-	  if (FD_ISSET (trace_icmp_sock (trace), &readset))
+	  if (FD_ISSET (fd, &readset))
 	    {
+	      int rc, type, code;
+
 	      triptime = ((double) now.tv_sec) * 1000.0 +
 		((double) now.tv_usec) / 1000.0;
 
-	      if (trace_read (trace))
+	      rc = trace_read (trace, &type, &code);
+
+	      if (rc < 0)
 		{
 		  /* FIXME: printf ("Some error ocurred\n"); */
 		  tries--;
@@ -291,10 +429,17 @@ do_try (trace_t * trace, const int hop,
 	      else
 		{
 		  if (tries == 0 || prev_addr != trace->from.sin_addr.s_addr)
-		    printf (" %s (%s) ",
-			    inet_ntoa (trace->from.sin_addr),
+		    {
+		      printf (" %s ", inet_ntoa (trace->from.sin_addr));
+		      if (opt_resolve_hostnames)
+			printf ("(%s) ",
 			    get_hostname (&trace->from.sin_addr));
-		  printf ("%.3fms ", triptime);
+		    }
+		  printf (" %.3fms ", triptime);
+
+		  /* Additional messages.  */
+		  if (rc > 0 && type == ICMP_DEST_UNREACH)
+		    printf ("!%c ", unreach_sign[code & 0x0f]);
 
 		}
 	      prev_addr = trace->from.sin_addr.s_addr;
@@ -309,13 +454,11 @@ do_try (trace_t * trace, const int hop,
 char *
 get_hostname (struct in_addr *addr)
 {
-  if (opt_resolve_hostnames)
-    {
-      struct hostent *info =
+  struct hostent *info =
 	gethostbyaddr ((char *) addr, sizeof (*addr), AF_INET);
-      if (info != NULL)
-	return info->h_name;
-    }
+  if (info != NULL)
+    return info->h_name;
+
   return inet_ntoa (*addr);
 }
 
@@ -323,13 +466,15 @@ void
 trace_init (trace_t * t, const struct sockaddr_in to,
 	    const enum trace_type type)
 {
+  int fd;
   const int *ttlp;
+
   assert (t);
   ttlp = &t->ttl;
 
   t->type = type;
   t->to = to;
-  t->ttl = TRACE_TTL;
+  t->ttl = opt_ttl;
 
   if (t->type == TRACE_UDP)
     {
@@ -368,6 +513,20 @@ trace_init (trace_t * t, const struct sockaddr_in to,
     {
       /* FIXME: type according to RFC 1393 */
     }
+
+  fd = (t->type == TRACE_UDP ? t->udpfd : t->icmpfd);
+
+  if (opt_tos >= 0)
+    if (setsockopt (fd, IPPROTO_IP, IP_TOS,
+		    &opt_tos, sizeof (opt_tos)) < 0)
+      error (0, errno, "setsockopt(IP_TOS)");
+
+#ifdef IP_OPTIONS
+  if (len_ip_opts)
+    if (setsockopt (fd, IPPROTO_IP, IP_OPTIONS,
+		    &ip_opts, len_ip_opts) < 0)
+      error (0, errno, "setsockopt(IPOPT_LSRR)");
+#endif /* IP_OPTIONS */
 }
 
 void
@@ -380,11 +539,20 @@ trace_port (trace_t * t, const unsigned short int port)
     t->to.sin_port = port;
 }
 
+/* Returned packet may contain, according to specifications:
+ *
+ *   IP-header + IP-options		(new IP-header)
+ *     + ICMP-header + old-IP-header	(ICMP message)
+ *     + old-IP-options + old-ICMP-header
+ */
+
+#define CAPTURE_LEN (MAXIPLEN + MAXICMPLEN)
+
 int
-trace_read (trace_t * t)
+trace_read (trace_t * t, int * type, int * code)
 {
-  int len;
-  unsigned char data[56];		/* For a TIME_EXCEEDED datagram. */
+  int len, rc = 0;
+  unsigned char data[CAPTURE_LEN];
   struct ip *ip;
   icmphdr_t *ic;
   socklen_t siz;
@@ -393,12 +561,16 @@ trace_read (trace_t * t)
 
   siz = sizeof (t->from);
 
-  len = recvfrom (t->icmpfd, (char *) data, 56, 0,
+  len = recvfrom (t->icmpfd, (char *) data, sizeof (data), 0,
 		  (struct sockaddr *) &t->from, &siz);
   if (len < 0)
     error (EXIT_FAILURE, errno, "recvfrom");
 
-  icmp_generic_decode (data, 56, &ip, &ic);
+  icmp_generic_decode (data, sizeof (data), &ip, &ic);
+
+  /* Pass type and code of incoming packet.  */
+  *type = ic->icmp_type;
+  *code = ic->icmp_code;
 
   switch (t->type)
     {
@@ -406,41 +578,67 @@ trace_read (trace_t * t)
       {
 	unsigned short *port;
 	if ((ic->icmp_type != ICMP_TIME_EXCEEDED
-	     && ic->icmp_type != ICMP_DEST_UNREACH)
-	    || (ic->icmp_type == ICMP_DEST_UNREACH
-		&& ic->icmp_code != ICMP_PORT_UNREACH))
+	     && ic->icmp_type != ICMP_DEST_UNREACH))
 	  return -1;
 
 	/* check whether it's for us */
-        port = (unsigned short *) &ic->icmp_ip + 11;
-	if (*port != t->to.sin_port)
+        port = (unsigned short *) ((void *) &ic->icmp_ip +
+			(ic->icmp_ip.ip_hl << 2) + sizeof (in_port_t));
+	if (*port != t->to.sin_port)	/* Network byte order!  */
 	  return -1;
 
-	if (ic->icmp_code == ICMP_PORT_UNREACH)
+	if (ic->icmp_type == ICMP_DEST_UNREACH)
 	  /* FIXME: Ugly hack. */
 	  stop = 1;
+
+	/* Only ICMP_PORT_UNREACH is an expected reply,
+	 * all other denials produce additional information.
+	 */
+	if (ic->icmp_type == ICMP_DEST_UNREACH
+	    && ic->icmp_code != ICMP_PORT_UNREACH)
+	  rc = 1;
       }
       break;
 
     case TRACE_ICMP:
-      if (ic->icmp_type != ICMP_TIME_EXCEEDED
-	  && ic->icmp_type != ICMP_ECHOREPLY)
+      if (! (ic->icmp_type == ICMP_TIME_EXCEEDED
+	     || ic->icmp_type == ICMP_ECHOREPLY
+	     || ic->icmp_type == ICMP_DEST_UNREACH) )
 	return -1;
 
       if (ic->icmp_type == ICMP_ECHOREPLY
-	  && (ic->icmp_seq != pid || ic->icmp_id != pid))
+	  && (ntohs (ic->icmp_seq) != seqno
+	      || ntohs (ic->icmp_id) != pid))
 	return -1;
-      else if (ic->icmp_type == ICMP_TIME_EXCEEDED)
+
+      if (ic->icmp_type == ICMP_TIME_EXCEEDED
+	  || ic->icmp_type == ICMP_DEST_UNREACH)
 	{
-	  unsigned short *seq = (unsigned short *) &ic->icmp_ip + 12;
-	  unsigned short *ident = (unsigned short *) &ic->icmp_ip + 13;
-	  if (*seq != pid || *ident != pid)
+	  unsigned short seq, ident;
+	  struct ip *old_ip;
+	  icmphdr_t *old_icmp;
+
+	  old_ip = (struct ip *) &ic->icmp_ip;
+	  old_icmp = (icmphdr_t *) ((void *) old_ip + (old_ip->ip_hl <<2));
+	  seq = ntohs (old_icmp->icmp_seq);
+	  ident = ntohs (old_icmp->icmp_id);
+
+	  /* An expired packet tests identity and sequence number,
+	   * whereas an undeliverable packet only checks identity.
+	   */
+	  if (ident != pid
+	      || (ic->icmp_type == ICMP_TIME_EXCEEDED
+		  && seq != seqno))
 	    return -1;
 	}
 
-      if (ip->ip_src.s_addr == dest.sin_addr.s_addr)
+      if (ip->ip_src.s_addr == dest.sin_addr.s_addr
+	  || ic->icmp_type == ICMP_DEST_UNREACH)
 	/* FIXME: Ugly hack. */
 	stop = 1;
+
+      if (ic->icmp_type == ICMP_DEST_UNREACH)
+	rc = 1;
       break;
 
       /* FIXME: Type according to RFC 1393. */
@@ -449,7 +647,7 @@ trace_read (trace_t * t)
       break;
     }
 
-  return 0;
+  return rc;
 }
 
 int
@@ -464,6 +662,7 @@ trace_write (trace_t * t)
     case TRACE_UDP:
       {
 	char data[] = "SUPERMAN";
+
 	len = sendto (t->udpfd, (char *) data, sizeof (data),
 		      0, (struct sockaddr *) &t->to, sizeof (t->to));
 	if (len < 0)
@@ -485,8 +684,10 @@ trace_write (trace_t * t)
     case TRACE_ICMP:
       {
 	icmphdr_t hdr;
-	/* FIXME: We could use the pid as the icmp seqno/ident. */
-	if (icmp_echo_encode ((unsigned char *) &hdr, sizeof (hdr), pid, pid))
+
+	/* The sequence number is updated to a valid value!  */
+	if (icmp_echo_encode ((unsigned char *) &hdr, sizeof (hdr),
+			      pid, ++seqno))
 	  return -1;
 
 	len = sendto (t->icmpfd, (char *) &hdr, sizeof (hdr),
@@ -550,3 +751,80 @@ trace_inc_port (trace_t * t)
   if (t->type == TRACE_UDP)
     t->to.sin_port = htons (ntohs (t->to.sin_port) + 1);
 }
+
+void
+trace_ip_opts (struct sockaddr_in *to)
+{
+#ifdef IP_OPTIONS
+  if (opt_gateways && *opt_gateways)
+    {
+      char *gateway, *optbase;
+      struct addrinfo hints, *res;
+
+      memset (&hints, 0, sizeof (hints));
+      hints.ai_family = AF_INET;
+      hints.ai_socktype = SOCK_DGRAM;
+
+      memset (&ip_opts, 0, sizeof (ip_opts));
+      optbase = ip_opts;
+
+      /* Set up any desired options.  Keep
+       * `optbase' updated, pointing to the
+       * part presently under construction.
+       */
+
+      /* 1. Loose source routing.  */
+      gateway = opt_gateways;
+      optbase[IPOPT_OPTVAL] = IPOPT_LSRR;
+      optbase[IPOPT_OLEN] = IPOPT_MINOFF - 1;	/* No payload yet.  */
+      optbase[IPOPT_OFFSET] = IPOPT_MINOFF;	/* Empty payload.  */
+
+      /* Traverse the gateway list, inserting
+       * addresses in the stated order.  Take
+       * care not to overflow available space.
+       */
+      while (gateway && *gateway
+	     && optbase[IPOPT_OFFSET]
+		< (int) (MAX_IPOPTLEN - sizeof (struct in_addr)))
+	{
+	  int rc;
+	  char *p;
+
+	  p = strpbrk (gateway, " ,;:");
+	  if (p)
+	    *p++ = '\0';
+
+	  rc = getaddrinfo (gateway, NULL, &hints, &res);
+	  if (rc)
+	    error (EXIT_FAILURE, errno, "gateway `%s' %s",
+		   gateway, gai_strerror(rc));
+
+	  /* Put target into next unused slot.  */
+	  memcpy (optbase + optbase[IPOPT_OLEN],
+		  &((struct sockaddr_in *) res->ai_addr)->sin_addr,
+		  sizeof (struct in_addr));
+
+	  freeaddrinfo (res);
+
+	  /* Option gained in length.  */
+	  optbase[IPOPT_OLEN] += sizeof (struct in_addr);
+
+	  gateway = p;
+	}
+
+      if (gateway && *gateway)
+	error (EXIT_FAILURE, 0, "too many gateways specified");
+
+      /* Append the final destination.  */
+      memcpy (optbase + optbase[IPOPT_OLEN],
+	      &to->sin_addr, sizeof (to->sin_addr));
+      optbase[IPOPT_OLEN] += sizeof (to->sin_addr);
+
+      /* 2. There is an implicit IPOPT_EOL after
+       * IPOPT_LSRR, ensured by the call to memset().
+       * Use it!
+       */
+      len_ip_opts = optbase[IPOPT_OLEN] + 1;
+    }
+#endif /* IP_OPTIONS */
+}
